local m, s, o
local rule_opt = {}
local redir_opt = {}
local vpn_type = "IPsec"

local uci = require("luci.model.uci").cursor()
local util_tlt = require "luci.util_tlt"

rule_opt[1] = {
	name 		= "Allow-"..vpn_type.."-ESP",
	target 		= "ACCEPT",
	src  		= "wan",
	proto 		= "esp",
	vpn_type	= vpn_type
}

rule_opt[2] = {
	name 		= "Allow-"..vpn_type.."-NAT-T",
	target 		= "ACCEPT",
	src  		= "wan",
	proto 		= "udp",
	dest_port	= "4500",
	vpn_type	= vpn_type
}

rule_opt[3] = {
	name 		= "Allow-"..vpn_type.."-IKE",
	target 		= "ACCEPT",
	src  		= "wan",
	dest_port	= "500",
	proto 		= "udp",
	vpn_type	= vpn_type
}

rule_opt[4] = {
	name 		= "Allow-"..vpn_type.."-Forward",
	target 		= "ACCEPT",
	src  		= "wan",
	proto 		= "all",
	dest 		= "*",
	extra 		= '-m policy --dir in --pol ipsec',
	vpn_type	= vpn_type
}

redir_opt[1] = {
	name		= "Exclude-"..vpn_type.."-from-NAT",
	proto		= 'any',
	extra		= '-m policy --dir out --pol ipsec',
	dest		= 'wan',
	target 		= "ACCEPT",
	vpn_type	= vpn_type
}

m = Map("ipsec")

s = m:section( TypedSection, "remote", translate("IPsec Configuration"), translate("This section displays IPsec instances currently existing on the router. In order to begin editing an instance, click the button that looks like a pencil located next to it."))
s.addremove = true
s.noname = true
s.add_title = "Add new ipsec configuration"
s.template = "cbi/tblsection"
s.instance_description = translate("This section is used to create new IPsec instances. To add a new instance, enter a custom name for it and click the 'Add' button.")
s.delete_alert = true
s.novaluetext = translate("There are no IPsec configurations yet")
s.extedit = luci.dispatcher.build_url("admin", "services", "vpn", "ipsec", "%s")
s.defaults = {enabled = "0"}

function s.create(self, section)
	local stat
	local created = TypedSection.create(self, section)
	if created then
		--Create additional sections
		stat = section:match("^[%w_]+$") and self.map:set(section.."_c", nil, "connection")
		stat = section:match("^[%w_]+$") and self.map:set(section.."_ph1", nil, "proposal")
		stat = section:match("^[%w_]+$") and self.map:set(section.."_ph2", nil, "proposal")

		self.map:set(section, "crypto_proposal", section.."_ph1")
		self.map:set(section.."_c", "crypto_proposal", section.."_ph2")

		--self.map:success_msg(translate("New IPsec instance created successfully. Configure it now"))
		return true
	end

	self.map:error_msg(translate("Failed to create new IPsec instance"))
	return false
end

function s.remove(self, section)
	local interface_name = self.map:get(section.."_c")
	uci:delete("network", interface_name['bind_to'], "auto")

	self.map.uci:foreach("ipsec", "proposal",
		function (s)
			if s[".name"] == section.."_ph1" or s[".name"] == section.."_ph2" then
				self.map:del(s[".name"])
			end
		end)

	self.map.uci:foreach("ipsec", "connection",
		function (s)
			if s[".name"] == section.."_c" then
				self.map:del(s[".name"])
			end
		end)

	return self.map:del(section)
end

local name = s:option( DummyValue, "name", translate("Tunnel name"), translate("Name of the tunnel. Used for easier tunnels management purpose only"))

function name.cfgvalue(self, section)
return section or "Unknown"
end

o = s:option( DummyValue, "gateway", translate("Remote VPN endpoint"), translate("Domain name or IP address. Any means waiting for incomming connections"))
o.cfgvalue = function (self, section)
	return self.map:get(section, self.option) or "any"
end

o = s:option( DummyValue, "type", translate("Type"), translate("The type of the connection."))
o.cfgvalue = function (self, section)
	return self.map:get(section.."_c", self.option) or "-"
end

o = s:option( DummyValue, "keyexchange", translate("Key exchange"), translate("Method of key exchange, which protocol should be used to initialize the connection"))
o.cfgvalue = function (self, section)
	return self.map:get(section.."_c", self.option) or "-"
end

o = s:option( DummyValue, "local_subnet", translate("Local subnet"), translate("Private subnet behind the left participant, expressed as network/netmask"))
o.cfgvalue = function (self, section)
	local subnet, k, v
	local value = self.map:get(section.."_c", self.option)
	if type(value) == "table" then
		for k, v in pairs(value) do
			if k == 1 then
				subnet = v
			else
				subnet = subnet..", "..v
			end
		end
		return subnet or "-"
	end
	return self.map:get(section.."_c", self.option) or "-"
end

o = s:option( DummyValue, "remote_subnet", translate("Remote subnet"), translate("Private subnet behind the right participant, expressed as network/netmask"))
o.cfgvalue = function (self, section)
	local subnet, k, v
	local value = self.map:get(section.."_c", self.option)
	if type(value) == "table" then
		for k, v in pairs(value) do
			if k == 1 then
				subnet = v
			else
				subnet = subnet..", "..v
			end
		end
		return subnet
	end
	return self.map:get(section.."_c", self.option) or "-"
end

s:option(Flag, "enabled", translate("")).last = true

m.on_commit = function (self)
	local enabled  = false
	local exists  = false
	local enabled_redir  = false
	local sid

	uci:foreach("ipsec", "remote", function(s)
		if s.enabled == "1" then
			enabled = true
		end
	end)

	uci:foreach("firewall", "redirect", function(s)
		if s.vpn_type == vpn_type then
			exists  = true
			sid = s[".name"]
			if s.enabled_redir == "1" then
				enabled_redir = true
			end
		end
	end)

	if enabled then
		for i=1,#rule_opt do
			luci.util_tlt.enable_vpn_firewall(vpn_type, nil, nil, rule_opt[i])
		end

		if not exists then
			for i=1,#redir_opt do
				uci:section("firewall", "redirect", nil, redir_opt[i])
			end
		elseif not enabled_redir then
			uci:set("firewall", sid, "enabled", "1")
		end
	else
		if vpn_type then luci.util_tlt.disable_vpn_firewall(vpn_type) end
	end
end

return m
